/*
 * tag: mod_auth_tkt defs
 */

#ifndef __MOD_AUTH_TKT
#define __MOD_AUTH_TKT

#define AUTH_COOKIE_NAME "TKTAuth"
#define BACK_ARG_NAME "back"
#define MD5andTSTAMP (MD5_DIGEST_SZ + 8)
#define SEPARATOR '!'
#define SEPARATOR_HEX "%21"
#define REMOTE_USER_DATA_SEPARATOR ':'
#define REMOTE_USER_DATA_TAG "tkt_user_data"
#define REMOTE_USER_DATA_ENV "REMOTE_USER_DATA"
#define REMOTE_USER_TOKENS_ENV "REMOTE_USER_TOKENS"

#define TKT_AUTH_VERSION "1.3.10"


/* per-directory configuration  */
typedef struct  {
  char *directory;
  char *login_url;
  char *timeout_url;
  char *unauth_url;
  char *auth_domain;
  array_header *auth_token;
  char *ignore_ip;
  char *auth_cookie_name;
  char *back_cookie_name;
  char *back_arg_name;
  unsigned long timeout_sec;
  double timeout_refresh;

#ifdef IFXAUTH
  char *dbhost;
  char *dbport;
  char *dbuser;
  char *dbpasswd;
  char *dboptions;
  char *dbname;
#endif
} tkt_auth_dir_conf;


typedef struct {
  char *md5_key;
  char *docroot;

} tkt_auth_serv_conf;


typedef struct tkt_cookie_struct 
{
  char *uid;
  char *tokens;
  char *user_data;
  unsigned int timestamp;
} tkt_cookie;


typedef struct {
  request_rec *r;
  char *cookie;
  char *cookie_name;
} cookie_res;


/* This kludge allows to use apxs which prefers to deal with just one file */
#include "tkt_digest.c"			  

#ifdef IFXAUTH
#include "ifx_auth.c"			  
#endif 


void tkt_auth_init (server_rec *s, pool *p);
static void * create_tkt_auth_config(pool *p, char* path);
static void * create_tkt_auth_serv_config(pool *p, server_rec* s);
static const char * setup_md5key (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * set_tkt_auth_token(cmd_parms *cmd, void *cfg, char *param);
static const char * set_tkt_auth_timeout(cmd_parms *cmd, void *cfg, char *param);
static const char * set_tkt_auth_timeout_refresh(cmd_parms *cmd, void *cfg, char *param);
static int authenticate_tkt_user (request_rec *r);

#ifdef IFXAUTH
static const char * setup_dbhost (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * setup_dbport (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * setup_dbuser (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * setup_dbpasswd (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * setup_dbname (cmd_parms *cmd, void *dir_cfg, char *param);
static const char * setup_dboptions (cmd_parms *cmd, void *dir_cfg, char *param);
#endif


/* ********************************************************************* *
 * parse_cookie() 
 populate struct tkt_cookie with values extracted from the cookie string
 if cookie string *magic is Base64 encoded, decode the cookie
 if the cookie does not appear to be in expected format, return 1
 on success return 0
 * ********************************************************************* */
static int parse_cookie(request_rec *r, char **magic, tkt_cookie *parsed);

/* ********************************************************************* *
 * find_our_cookie() and cookie_match()
 find TKT_AUTH cookie among other cookies
 cookie_match() is a helper function called by ap_table_do() for each 
 cookie in a request. If TKT_AUTH cookie is found, it's value is returned 
 by find_our_cookie()
 If cookie is not found find_our_cookie() returns NULL
 * ********************************************************************* */
static int cookie_match(void *result, const char *key, const char *cook); 

static char * find_our_cookie(request_rec *r, tkt_auth_dir_conf *tkt_cfg);
static char * find_our_url(request_rec *r, tkt_auth_dir_conf *tkt_cfg);


/* ********************************************************************* *
 * c2x() and escape_str() 
 these functions should not be here
 Apache's ap_escape_path etc, does not seem to work for '=' and '&' 
 on Linux So here we go...
 * ********************************************************************* */
static ap_inline unsigned char *c2x(unsigned what, unsigned char *where);
static char * escape_str(pool *p, const char *segment);


/* ********************************************************************* *
 * redirect()
 set cookie with currently accessed URL and redirect the request 
 * ********************************************************************* */
static int redirect(request_rec *r, char *location, tkt_auth_dir_conf *tkt_cfg);


/* ********************************************************************* *
 * add_query_string()

 Put user data into query string, so that it could be used by 
 servlets, etc.
 If query string in request already has REMOTE_USER_DATA_TAG=somevalue, 
 it is replaced with authenticated user data
 * ********************************************************************* */
static void add_query_string(request_rec *r, char *data);

/* ********************************************************************* 
 * remove all occurrences of strip=value from the request query string,
 *   returning the result
 * ********************************************************************* */
static char * query_strip(request_rec *r, const char *strip);

#endif

